from typing import Optional
import pandas as pd
from OctopusAgile import Agile
import warnings

from .abstract import PricingCollector
from .misc import iso8601z


class Octopus(PricingCollector):
    country = 'UK'
    supplier = 'Octopus'
    timezone = 'Europe/London'

    def __init__(self, region_code: str, **kwargs):
        self.agile = Agile(region_code)
        self.price_name = f'{self.supplier}_{region_code}'
        super(Octopus, self).__init__(**kwargs)

    def get_data(
            self,
            interval: Optional[pd.Interval] = None,
    ) -> pd.DataFrame:
        if not interval:
            rates = self.agile.get_new_rates()
        else:
            date_from = iso8601z(interval.left)
            date_to = iso8601z(interval.right)
            if interval.length.days > 2:
                warnings.warn(
                    'You requested an interval larger than 2 days, '
                    'be advised that the response may not be complete')
            rates = self.agile.get_rates(date_from, date_to)

        rates = pd.DataFrame.from_dict(rates['date_rates'], orient='index')
        # noinspection PyTypeChecker
        rates.index = pd.to_datetime(rates.index)
        rates.rename(columns={0: 'value_inc_vat'}, inplace=True)
        rates = rates.tz_convert(self.timezone)
        rates.sort_index(inplace=True)
        return rates



import pandas as pd


def iso8601z(timestamp: pd.Timestamp) -> str:
    """
    Convert Timestamp to ISO8601-notation in UTC with "Z" suffix
    """
    timestamp = timestamp.tz_convert('UTC')
    iso = timestamp.strftime('%Y-%m-%dT%H:%M:%SZ')
    return iso



from abc import ABC
from enum import Enum
from typing import Optional

import pandas as pd


class CollectorType(Enum):
    """Enum of types of collectors"""
    PRICE = 'price'


class Collector(ABC):
    """
    Abstract class.
    Represents a connection to an external service where data is gathered.
    """
    def get_data(
            self,
            interval: Optional[pd.Interval] = None,
    ) -> pd.DataFrame:
        """
        Method to request data and return it as Pandas DataFrame with
        DateTimeIndex.

        If no interval is specified, "current" data (eg. "today"), should be
        returned.
        Dates should never be timezone naive.

        If a geographic region is known, the returned DataFrame should have
        a timezone-localised index.
        """
        raise NotImplementedError('To be implemented by subclass')


class PricingCollector(Collector, ABC):
    """Abstract collector for pricing data"""
    country: str = NotImplemented
    supplier: str = NotImplemented
    timezone: str = NotImplemented



from typing import Dict

from .abstract import CollectorType, Collector
from .pricing import Octopus


def create_collector(collector_type: CollectorType, body: Dict) -> Collector:
    if collector_type == CollectorType.PRICE:
        supplier = body['supplier']
        if supplier == 'Octopus':
            region_code = body['regionCode']
            collector = Octopus(region_code=region_code)
        else:
            raise LookupError(f'Supplier {supplier} not supported, or unknown')
    else:
        raise NotImplementedError(f'Collector Type {collector_type} not yet '
                                  f'supported')
    return collector
